module Element.Keyed exposing (column, el, row)

{-| Notes from the `Html.Keyed` on how keyed works:

    A keyed node helps optimize cases where children are getting added, moved, removed, etc. Common examples include:

        - The user can delete items from a list.
        - The user can create new items in a list.
        - You can sort a list based on name or date or whatever.

    When you use a keyed node, every child is paired with a string identifier. This makes it possible for the underlying diffing algorithm to reuse nodes more efficiently.

This means if a key is changed between renders, then the diffing step will be skipped and the node will be forced to rerender.

@docs el, column, row

-}

import Element exposing (Attribute, Element, fill, height, width)
import Internal.Model as Internal


{-| -}
el : List (Attribute msg) -> ( String, Element msg ) -> Element msg
el attrs child =
    Internal.element Internal.NoStyleSheet
        Internal.asEl
        Nothing
        (width Element.shrink
            :: height Element.shrink
            -- :: centerY
            :: Element.centerX
            :: Internal.Class "x-content-align" "content-center-x"
            :: Internal.Class "y-content-align" "content-center-y"
            :: attrs
        )
        (Internal.Keyed [ child ])


{-| -}
row : List (Attribute msg) -> List ( String, Element msg ) -> Element msg
row attrs children =
    Internal.element
        Internal.NoStyleSheet
        Internal.asRow
        Nothing
        (Internal.Class "x-content-align" "content-center-x"
            :: Internal.Class "y-content-align" "content-center-y"
            :: width fill
            :: attrs
        )
        (Internal.Keyed <| Internal.keyedRowEdgeFillers children)


{-| -}
column : List (Attribute msg) -> List ( String, Element msg ) -> Element msg
column attrs children =
    Internal.element Internal.NoStyleSheet
        Internal.asColumn
        Nothing
        (Internal.Class "y-content-align" "content-top"
            :: Internal.Class "x-content-align" "content-center-x"
            :: height fill
            :: width fill
            :: attrs
        )
        (Internal.Keyed <| Internal.keyedColumnEdgeFillers children)
